const std = @import("std");

pub const c = @cImport({
    @cInclude("assimp/cimport.h");
    @cInclude("assimp/scene.h");
    @cInclude("assimp/postprocess.h");
});

/// Load the texture material names in a scene.
/// Don't forget to free each element after use.
pub fn loadMaterials(allocator: std.mem.Allocator, scene: *const c.aiScene) !std.ArrayList(?[]const u8) {
    // Create 1:1 sized list of textures
    var texture_list = try std.ArrayList(?[]const u8).initCapacity(allocator, scene.mNumMaterials);

    // Go through each material and copy its texture file name (if it exists)
    for (0..scene.mNumMaterials) |i| {
        // Get the material
        const material = scene.mMaterials[i];

        // Initialise the texture to empty string (will be replaced if the texture exists)
        // try texture_list.append("");

        // Check for diffuse texture (standard detail texture)
        if (c.aiGetMaterialTextureCount(material, c.aiTextureType_DIFFUSE) != 0) {
            // Get the path of the texture file
            var path: c.aiString = undefined;
            if (c.aiGetMaterialTexture(
                material,
                c.aiTextureType_DIFFUSE,
                0,
                &path,
                null,
                null,
                null,
                null,
                null,
                null,
            ) == c.AI_SUCCESS) {
                // Cut of any directory information already present
                var it = std.mem.splitBackwardsAny(u8, &path.data, "\\/");
                if (it.next()) |filename| {
                    texture_list.appendAssumeCapacity(try allocator.dupe(u8, filename));
                }
            } else {
                texture_list.appendAssumeCapacity(null);
            }
        } else {
            texture_list.appendAssumeCapacity(null);
        }
    }

    return texture_list;
}
